#pragma once

#include "cudaMain.cuh"
const double PI = 3.141592653589793;

class GameCharactor
{
public:
	GameCharactor() {
		x = y = 0;
		near_id = -1;
	}

	float x;
	float y;
	float w;
	float direction;

	int near_id;
};

class GameField
{
public:
	std::vector<GameCharactor> chars;
	int ms;
	int count;
	float* pBufX, * pBufY;
	int* pBufResult;

	GameField() {
		count = 0;
		pBufX = pBufY = NULL;
		pBufResult = NULL;
	}

	~GameField() {
		if (count != 0) {
			free(pBufX);
			free(pBufY);
			free(pBufResult);
		}
	}

	int make_random_charactors(int n) {
		chars.clear();
		chars.resize(n);
		srand((unsigned int)time(0));
		for (int i = 0; i < n; i++) {
			chars[i].x = (float)rand() / RAND_MAX;
			chars[i].y = (float)rand() / RAND_MAX;
			chars[i].w = 0.01f;
			chars[i].direction = (float)rand() / RAND_MAX * 360;
		}

		// CUDAp̃obt@m
		count = n;
		pBufX = (float *)malloc(sizeof(float) * count);
		pBufY = (float*)malloc(sizeof(float) * count);
		pBufResult = (int*)malloc(sizeof(int) * count);

		return 0;
	}

	int move_charactors() {
		int n = chars.size();
		for (int i = 0; i < n; i++) {
			GameCharactor& a = chars[i];
			a.x += 0.001 * cosf(a.direction * PI / 180);
			a.y += 0.001 * sinf(a.direction * PI / 180);
		}
		return 0;
	}

	// SIuWFNgAԋ̋߂_vZ
	int calc_near_id() {
		int start = clock();
		int n = chars.size();
		float dist, d;
		for (int i = 0; i < n; i++) {
			dist = 10000;
			for (int j = 0; j < n; j++) {
				if (i == j)
					continue;
				d = sqrtf(
					powf(chars[i].x - chars[j].x, 2) +
					powf(chars[i].y - chars[j].y, 2) );
				if (dist > d) {
					dist = d;
					chars[i].near_id = j;
				}
			}
		}
		ms = clock() - start;
		return true;
	}

	// SIuWFNgAԋ̋߂_vZ(CUDAgp
	int calc_near_id_cuda() {
		int start = clock();

		for (int i = 0; i < count; i++) {
			pBufX[i] = chars[i].x;
			pBufY[i] = chars[i].y;
		}
		cuda_near_search(pBufX, pBufY, pBufResult, count);

		for (int i = 0; i < count; i++) 
			chars[i].near_id = pBufResult[i];		

		ms = clock() - start;
		return 0;
	}
};

class GameData
{
public:
	GameField field;

};

