#include <stdio.h>

#include "cuda_runtime.h"
#include "device_launch_parameters.h"
#include "cudaMain.cuh"

__global__ void gpu_near_search(float* dev_x, float* dev_y, int* results, int cnt)
{
	int idx = blockIdx.x * blockDim.x + threadIdx.x;	// CfbNXvZ
	int i;
	float dist = 10000, d;
	for (i = 0; i < cnt; i++) {
		if (idx == i)
			continue;
		d = sqrtf(
			powf(dev_x[idx] - dev_x[i], 2) +
			powf(dev_y[idx] - dev_y[i], 2));
		if (dist > d) {
			dist = d;
			results[idx] = i;
		}
	}
}

int cuda_near_search(float* host_x, float* host_y, int* results, int cnt) {
	memset(results, -1, cnt);

	float* dev_x, * dev_y;
	int* dev_results;
	cudaMalloc(&dev_x, cnt * sizeof(float));
	cudaMalloc(&dev_y, cnt * sizeof(float));
	cudaMalloc(&dev_results, cnt * sizeof(int));

	cudaMemcpy(dev_x, host_x, cnt * sizeof(float), cudaMemcpyHostToDevice);
	cudaMemcpy(dev_y, host_y, cnt * sizeof(float), cudaMemcpyHostToDevice);
	cudaMemcpy(dev_results, results, cnt * sizeof(int), cudaMemcpyHostToDevice);

	gpu_near_search <<< (cnt + 255) / 256, 256 >>> (dev_x, dev_y, dev_results, cnt);

	cudaMemcpy(results, dev_results, cnt * sizeof(int), cudaMemcpyDeviceToHost);
	return 0;
}